/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/
 
// Implementation of class Cmp3PlayListHandler

#include "mp3_playlist_handler.h"
#include "mp3_source_config.h"
#include <zz_cgi.h>

// Construction
Cmp3PlayListHandler::Cmp3PlayListHandler( void )
: CzzHTTPServer( Cmp3SourceConfig::getWebserverPort().c_str(), SOMAXCONN ), m_Log( Cmp3SourceConfig::appName().c_str() ), m_PlayLists()
{
	int size = 10240;
	char* sections_str = new char[size];
    CzzString tmp;
    memset( sections_str, 0, size );
    CzzString file = "..\\..\\mp3_webserver.ini";

    int r = GetPrivateProfileSection( "users", sections_str, size, file.c_str() );
    int i = 1;

    while( r == sizeof(sections_str) - 2 )
    {
        i++;
        delete [] sections_str;
        char* sections_str = new char[size*i];
        memset( sections_str, 0, size );
        r = GetPrivateProfileSection( "users", sections_str, size*i, file.c_str() );
    }

    if( r )
    {
		char* p = sections_str;

		for( i=0; i <= r; i++ )
		{
			if( sections_str[i] == 0 )
			{
				tmp = p;
            
				if( tmp.size() )
				{
					int pos = tmp.find( "=" );
					CzzString usr = tmp.substr( 0, pos );
					m_PlayLists[ usr ] = new Cmp3PlayList( usr );
				}

				p = sections_str + i + 1;
			}
		}
	}

    delete [] sections_str;
}

// Destruction
Cmp3PlayListHandler::~Cmp3PlayListHandler()
{
	map<CzzString, Cmp3PlayList* >::iterator i;

	for( i = m_PlayLists.begin(); i != m_PlayLists.end(); i++ )
    {
		delete i->second;
	}

	m_PlayLists.clear();
}

void Cmp3PlayListHandler::OnStart()
{

	while( !m_StopRequest ) 
	{
		listen();
	}
}

map<CzzString, Cmp3PlayList* >* Cmp3PlayListHandler::getPlayLists( void )
{
	return &m_PlayLists;
}

Cmp3PlayList* Cmp3PlayListHandler::getPlayList( const CzzString& usr )
{
	return m_PlayLists[usr];
}

// Listen for a request
int Cmp3PlayListHandler::listen( void )
{
    CzzHTTPRequest* request = NULL;
    int result = 0;

    try
    {
        ZZ_HTTP_CMD cmd;
        CzzString uri;
        
        request = waitForRequest();

        uri = request->getUri();
        cmd = request->getCommand();


		/*if( uri == "/favicon.ico" )
		{
			
		}
		else*/ if( uri.find( "mp3_radio.exe?svc=" ) != -1 )
		{
			int pos = uri.find( "?" );
			CzzString params = uri.substr( pos + 1, uri.size() - pos + 1 );

			// Stt upp milj fr CGI
			CzzString query = CzzString::Format( "QUERY_STRING=%s", params.c_str() );
			int r = 0;
			r = putenv( "REQUEST_METHOD=GET" );
			r = putenv( query.c_str() );
			CzzCGI cgi;
			CzzString svcstr = cgi[ "svc" ];
			CzzString usr = cgi[ "usr" ];
			int svc = atoi( svcstr.c_str() );
			
			switch( svc )
			{
				case 2:
					result = addSong( usr, request, cgi[ "name" ].c_str(), cgi[ "album" ].c_str(), cgi[ "song" ].c_str() );
					break;

				case 3:
					result = removeSong( usr, request, cgi[ "name" ].c_str(), cgi[ "album" ].c_str(), cgi[ "song" ].c_str() );
					break;
			
				case 4:
					result = list( usr, request );
					break;

				case 5:
					result = nowPlaying( usr, request );
					break;

				case 6:
					result = commingUp( usr, request );
					break;

				case 7:
					result = lastPlayed( usr, request );
					break;

				case 9:
					result = clear( usr, request );
					break;

				case 10:
					result = forward( usr, request );
					break;

				case 11:
					result = reverse( usr, request );
					break;

				case 12:
					result = replacePlaylist( usr, request, cgi[ "name" ].c_str() );
					break;

				case 13:
					result = randomize( usr, request );
					break;

				case 20:
					result = listIndexes( usr, request );
					break;

				default:
					result = serviceNotFound( request );
					break;
			}
		}
		else
			result = serviceNotFound( request );
    }
    catch( CzzException& e )
    {
        result = internalError( request, e.GetReason().c_str() );
    }
    catch( ... )
    {
        result = internalError( request, "Internal server error" );
    }

    return result;
}

int Cmp3PlayListHandler::replacePlaylist( const CzzString& usr, CzzHTTPRequest* req, const char* name )
{
    CzzBuffer buffer( 1024 );
    getPlayList(usr)->replacePlaylist( name );

	CzzString data = CzzString::Format( "Replaced playlist: %s", name  );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::clear( const CzzString& usr, CzzHTTPRequest* req )
{
	getPlayList(usr)->clear();

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Cleared" );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}


int Cmp3PlayListHandler::list( const CzzString& usr, CzzHTTPRequest* req )
{
    CzzBuffer buffer( 50240 );
    CzzString data = getPlayList(usr)->list();

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::listIndexes( const CzzString& usr, CzzHTTPRequest* req )
{
    CzzBuffer buffer( 50240 );
    CzzString data = getPlayList(usr)->listIndexes();

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::nowPlaying( const CzzString& usr, CzzHTTPRequest* req )
{
    CzzBuffer buffer( 1024 );
    CzzString data = getPlayList(usr)->nowPlaying();

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::commingUp( const CzzString& usr, CzzHTTPRequest* req )
{
    CzzBuffer buffer( 1024 );
    CzzString data = getPlayList(usr)->commingUp();

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::lastPlayed( const CzzString& usr, CzzHTTPRequest* req )
{
    CzzBuffer buffer( 1024 );
    CzzString data = getPlayList(usr)->lastPlayed();

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::addSong( const CzzString& usr, CzzHTTPRequest* req, const char* name, const char* album, const char* song )
{
	getPlayList(usr)->add( name, album, song );

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Added: %s/%s/%s", name, album, song );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::forward( const CzzString& usr, CzzHTTPRequest* req )
{
	getPlayList(usr)->forward();

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Forward one song" );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::reverse( const CzzString& usr, CzzHTTPRequest* req )
{
	getPlayList(usr)->reverse();

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Reverse one song" );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::randomize( const CzzString& usr, CzzHTTPRequest* req )
{
	getPlayList(usr)->randomize();

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Randomizing songlist" );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

int Cmp3PlayListHandler::removeSong( const CzzString& usr, CzzHTTPRequest* req, const char* name, const char* album, const char* song )
{
	getPlayList(usr)->remove( name, album, song );

    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "Removed: %s/%s/%s", name, album, song );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

// Service not found error reply
int Cmp3PlayListHandler::serviceNotFound( CzzHTTPRequest* req )
{
    CzzBuffer buffer( 1024 );
    CzzString data;

    data = "Not a valid service";

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_NOT_FOUND, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );
    
    return buffer.getSize();
}

// Internal error reply
int Cmp3PlayListHandler::internalError( CzzHTTPRequest* req, const char* msg )
{
    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "%s", msg );

    m_Log.Print( _DEBUGPARAMS, "Sending: '%s'", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( req, ZZ_HTTP_STATUS_INTERNAL_SERVER_ERROR, ZZ_HTTP_CONTENT_TYPE_TEXT, buffer );

    return buffer.getSize();
}

