/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3ArtistListData

#include "mp3_artist_list_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3ArtistListData::Cmp3ArtistListData( void )
: CzzDataService( "listArtistData" )
{

}

// Destruction
Cmp3ArtistListData::~Cmp3ArtistListData()
{

}

CzzDataServiceReply* Cmp3ArtistListData::execute( void* session, CzzParameterMap* req )
{
	CzzDataService::execute( session, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();

	reply->addFieldName( MP3_FLD_LIST_ART );
	reply->addFieldName( MP3_FLD_TITLE_ROW );
	reply->addFieldName( MP3_FLD_INFO_ROW );
	reply->addFieldName( MP3_FLD_TITLE_LINK );
	reply->addFieldName( MP3_FLD_PLAY_LINK );

	long handle = 0;
	int error = 0;
	int album = 0;
	bool numerical = false;
	_finddata_t finfo;
	CzzString path = Cmp3Config::getDataDir();
	CzzString subdir;
	CzzString anchor;
	CzzString cgi = Cmp3Config::getCGIDir();
	
	try{ anchor = (*req)[MP3_CGI_PAR_INDEX]; } catch( ... ) { anchor = "A"; }
	
	if( anchor.size() == 0 )
		anchor = "A";

	if( anchor == "0...9" )
		numerical = true;

	if( anchor == "AO" )
		anchor = "";

	if( anchor == "AE" )
		anchor = "";

	if( anchor == "OE" )
		anchor = "";
	
	// Find first file
	handle = _findfirst( (path + "\\*.*").c_str(), &finfo );
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( !numerical && (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) && finfo.name[0] == anchor[0] )
				{
					subdir = path + "\\" + finfo.name;
					album = countAlbums( subdir );
					
					reply->addFieldValue( CzzString::Format( "%s/%s/Folder.jpg", 
											Cmp3Config::getMusicRoot().c_str(), 
											urlEncode( finfo.name ).c_str() ) );

					reply->addFieldValue( finfo.name );
					reply->addFieldValue( CzzString::Format( "%d album", album ) );
					
					reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s", 
											Cmp3Config::getCGIDir().c_str(), 
											MP3_SVC_LIST_RECORDS, 
											urlEncode( finfo.name ).c_str() ) );

					reply->addFieldValue( CzzString::Format( "%s/mp3_playlist.exe?svc=%d&name=%s", 
											Cmp3Config::getCGIDir().c_str(), 
											0, 
											urlEncode( finfo.name ).c_str() ) );
				}
				else if( numerical && (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) && ( finfo.name[0] == '0' || 
						finfo.name[0] == '1' || finfo.name[0] == '2' || finfo.name[0] == '3' || 
						finfo.name[0] == '4' || finfo.name[0] == '5' || finfo.name[0] == '6' || 
						finfo.name[0] == '7' || finfo.name[0] == '8' || finfo.name[0] == '9' ) )
				{
					subdir = path + "\\" + finfo.name;
					album = countAlbums( subdir );
					
					reply->addFieldValue( CzzString::Format( "%s/%s/Folder.jpg", 
											Cmp3Config::getMusicRoot().c_str(), 
											urlEncode( finfo.name ).c_str() ) );

					reply->addFieldValue( finfo.name );
					reply->addFieldValue( CzzString::Format( "%d album", album ) );
					
					reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s", 
											Cmp3Config::getCGIDir().c_str(), 
											MP3_SVC_LIST_RECORDS, 
											urlEncode(finfo.name).c_str() ) );

					reply->addFieldValue( CzzString::Format( "%s/mp3_playlist.exe?svc=%d&name=%s", 
											Cmp3Config::getCGIDir().c_str(), 
											0, 
											urlEncode(finfo.name).c_str() ) );
				}
			}

			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	return reply;
}

int Cmp3ArtistListData::countAlbums( const CzzString& path )
{
	long handle = 0;
	int error = 0;
	int result = 0;
	_finddata_t finfo;

	// Find first file
	handle = _findfirst( (path + "\\*.*").c_str(), &finfo );

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				// Is it a subdir?
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					result++;
				}
			}

			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );

	return result;
}

int Cmp3ArtistListData::size()
{
    return sizeof(Cmp3ArtistListData);
}