/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3PlaylistContentRender

#include "mp3_http_requester.h"
#include "mp3_playlist_content_render.h"
#include "mp3_song_list_render.h"
#include "mp3_general_data.h"
#include "mp3_config.h"
#include "mp3_playlist_content_data.h"
#include "mp3_playlist_content_list_data.h"
#include "mp3_svc.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>
 

// Construction
Cmp3PlaylistContentRender::Cmp3PlaylistContentRender( void* s, CzzParameterMap* cgi, int svc )
: CzzOutputRender( CzzString::Format( ".\\%s\\%s\\renderPlaylist.html", Cmp3Config::getLang().c_str(), Cmp3Config::getSkin().c_str() ) , s, cgi )
{

	CzzString name = (*cgi)[ "name"];
	CzzString back_url = CzzString::Format( "%s/mp3_gui.exe?svc=%d", Cmp3Config::getCGIDir().c_str(), MP3_SVC_LIST_M3U );
    CzzString session = *((CzzString*)s);
    CzzString user = getenv( "REMOTE_USER" );

	if( svc == MP3_SVC_LIST_M3U_DELETE && name == "session" )
	{
		unlink( (".\\" + session + ".ini").c_str() );
		unlink( (".\\" + session + ".m3u").c_str() );
	}

    if( svc == MP3_SVC_LIST_M3U_DELETE && name != "session" )
	{
		throw CzzLogException( _LAF, 4711, CzzString::Format( "Not allowed for a saved static playlist" ) );
	}


	if( svc == MP3_SVC_LIST_M3U_SAVE && name == "session" )
	{
		struct tm* pt;
		time_t t;
		time( &t );                
		pt = localtime( &t );
		
		
		CzzString path = Cmp3Config::getM3uDir();

		CzzFile f( (".\\" + session + ".m3u").c_str() );

        if( !f.Exist() )
            throw CzzLogException( _LAF, 4711, CzzString::Format( "Not allowed for a saved static playlist" ) );

		name = CzzString::Format( "Mix %4d-%02d-%02d %02d.%02d.%02d - %s.m3u", 
					pt->tm_year+1900, pt->tm_mon+1, pt->tm_mday, 
					pt->tm_hour, pt->tm_min, pt->tm_sec, user.c_str() ); 

		CzzString to = CzzString::Format( "%s\\%s", path.c_str(), name.c_str() );

		f.CopyTo( to.c_str() );
		f.Close();
		
		unlink( (".\\" + session + ".ini").c_str() );
		unlink( (".\\" + session + ".m3u").c_str() );
	}

	if( svc == MP3_SVC_LIST_M3U_REMOVE && name == "session" )
	{
		CzzString line = (*cgi)["line"];
		char buff[2048];
		bool update_cache = false;

		FILE* f = fopen( (".\\" + session + ".m3u").c_str(), "r" );
		FILE* n = fopen( (".\\" + session + ".new").c_str(), "a" );

		while( fgets( buff, 2048, f ) != NULL )
		{
			if( strcmp( buff, (line + "\n" ).c_str() ) != 0 )
				fprintf( n, "%s", buff );
			else
			{
				if( update_cache == true )
					fprintf( n, "%s", buff );

				update_cache = true;
			}
		}

		fclose(f);
		fclose(n);

		unlink( (".\\" + session + ".m3u").c_str() );
		
		CzzFile ny( (".\\" + session + ".new").c_str() );
		ny.CopyTo( (".\\" + session + ".m3u").c_str() );
		ny.Close();

		unlink( (".\\" + session + ".new").c_str() );

		if( update_cache )
			Cmp3SongListRender::updateCache(session, false, line);
	}
	else if( svc == MP3_SVC_LIST_M3U_REMOVE && name != "session" )
	{
		throw CzzLogException( _LAF, 4711, CzzString::Format( "Not allowed for a saved static playlist" ) );
	}

	if( svc == MP3_SVC_LIST_RADIO_REPLACE )
	{
		Cmp3HttpRequester req;
		req.replaceRadioQueue( name.c_str() );
	}

	if( name == "session" )
	{
		this->addFieldSvc( new Cmp3GeneralData( back_url, "myMusic", 
			CzzString::Format( "%s - %s", Cmp3Config::HEADLINE_PLAYLIST_CONTENT().c_str(), Cmp3Config::HEADLINE_PLAYLIST_NOTSAVED().c_str() )) );
	}
	else
	{
		this->addFieldSvc( new Cmp3GeneralData( back_url, "myMusic", 
			CzzString::Format( "%s - %s", Cmp3Config::HEADLINE_PLAYLIST_CONTENT().c_str(), name.substr(0, name.size()-4).c_str() )) );
	}
	
	this->addFieldSvc( new Cmp3PlaylistContentData( name ) );
	this->addListSvc( MP3_FLD_TABLE, new Cmp3PlaylistContentListData( name ) );
}

// Destruction
Cmp3PlaylistContentRender::~Cmp3PlaylistContentRender()
{

}

