/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3RecordListData

#include "mp3_record_list_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3RecordListData::Cmp3RecordListData( void )
: CzzDataService( "listRecordData" )
{

}

// Destruction
Cmp3RecordListData::~Cmp3RecordListData()
{

}

CzzDataServiceReply* Cmp3RecordListData::execute( void* session, CzzParameterMap* req )
{

	CzzDataService::execute( session, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();
	long handle = 0;
	int error = 0;
	int result = 0;
	_finddata_t finfo;
	CzzString name = (*req)[ MP3_CGI_PAR_NAME ];
	CzzString path = Cmp3Config::getDataDir();

	reply->addFieldName( MP3_FLD_LIST_ART );
	reply->addFieldName( MP3_FLD_TITLE_ROW );
	reply->addFieldName( MP3_FLD_INFO_ROW );
	reply->addFieldName( MP3_FLD_TITLE_LINK );
	reply->addFieldName( MP3_FLD_PLAY_LINK );


	// Find first file
	handle = _findfirst( (path + "\\" + name + "\\*.*").c_str(), &finfo );

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				// Is it a subdir?
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					printRecordList( name, finfo.name, reply );
			}

			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
	
	return reply;
}

void Cmp3RecordListData::printRecordList( const CzzString& name, const char* album, CzzDataServiceReply* reply )
{
	CzzString cgi = Cmp3Config::getCGIDir();
	CzzString dir = Cmp3Config::getMusicRoot();
	CzzString path = Cmp3Config::getDataDir();
	CzzString root = Cmp3Config::getRootDir();
	CzzString year("-");
	long handle = 0;
	int error = 0;
	uint32 speed = 128;
	_finddata_t finfo;
	handle = _findfirst( (path + "\\" + name + "\\" + CzzString(album) + "\\*.mp3").c_str(), &finfo );

	// Read the mp3
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{

		ID3_Tag tag( (path + "\\" + name + "\\" + CzzString(album) + "\\" + CzzString(finfo.name)).c_str() );
		ID3_Frame* frame = NULL;
		ID3_Field* fld = NULL;

		frame = tag.Find( ID3FID_YEAR );

		if( frame != NULL )
		{
			fld = frame->GetField( ID3FN_TEXT );
    
			if( fld != NULL )
			{
				fld->SetEncoding(ID3TE_ISO8859_1);
				const char* p = fld->GetRawText();

				if( p != NULL && strlen(p) == 4 && ( p[0]== '1' || p[0]== '2') )
					year = p;
			}
		}
		
		const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();

		if( hdr != NULL )
		{
			if( hdr->bitrate != 0 )
				speed = hdr->bitrate/1000;
			if( hdr->vbr_bitrate != 0 )
				speed = hdr->vbr_bitrate/1000;
		}

		_findclose( handle );
	}


	/*
	reply->addFieldName( MP3_FLD_LIST_ART );
	reply->addFieldName( MP3_FLD_TITLE_ROW );
	reply->addFieldName( MP3_FLD_INFO_ROW );
	reply->addFieldName( MP3_FLD_TITLE_LINK );
	reply->addFieldName( MP3_FLD_PLAY_LINK );
	*/

	reply->addFieldValue( urlEncode( 
							CzzString::Format( "%s/%s/%s/Folder.jpg", 
								dir.c_str(), name.c_str(), album ).c_str() ) );

	reply->addFieldValue( album );

	reply->addFieldValue( CzzString::Format( "%s - %s [%d kbit/s]", name.c_str(), year.c_str(), speed ) );

	reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s&album=%s", 
								cgi.c_str(), MP3_SVC_LIST_SONGS, urlEncode(name).c_str(), urlEncode(album).c_str() ) );
	
	reply->addFieldValue( CzzString::Format( "%s/mp3_playlist.exe?svc=1&name=%s&album=%s", 
												cgi.c_str(), urlEncode(name).c_str(), urlEncode(album).c_str() ) );
}

int Cmp3RecordListData::size()
{
    return sizeof(Cmp3RecordListData);
}