/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3ThreadSafeConfig

#include "mp3_thread_safe_config.h"
#include <zz_log_exception.h>

CzzWinThreadMutex Cmp3ThreadSafeConfig::m_Mutex;

// Construction
Cmp3ThreadSafeConfig::Cmp3ThreadSafeConfig( const CzzString& file )
 : m_BlockList(), m_File(file)
{

}

// Destruction
Cmp3ThreadSafeConfig::~Cmp3ThreadSafeConfig()
{
	map<CzzString, Cmp3Block* >::iterator i;

	// Free the objects pointed to in the map
	for( i = m_BlockList.begin(); i != m_BlockList.end(); i++ )
		delete m_BlockList[ i->first ];

	// Free the maps
	m_BlockList.clear();
}

CzzString Cmp3ThreadSafeConfig::getEntry( const CzzString& block, const CzzString& entry )
{
	CzzString result;

	// Lock
	Cmp3ThreadSafeConfig::m_Mutex.Acquire( true );

	Cmp3Block* list = m_BlockList[block];

	if( list == NULL )
	{
		list = new Cmp3Block();
		m_BlockList[block] = list;
	}

	result = (*list)[entry];

	if( result.size() == 0 )
	{
		char var[ 256 ];

		if( !GetPrivateProfileString( block.c_str(), entry.c_str(), "", var, sizeof( var ), m_File.c_str() ) )
		{
			Cmp3ThreadSafeConfig::m_Mutex.Release();
			throw CzzLogException( _LAF, 4711, CzzString::Format( "Could'nt read configuration [%s] - '%s' from '%s'", 
					block.c_str(), entry.c_str(), m_File.c_str() ) );
		}

		(*list)[entry] = var;
		result = var;
	}
	else
		result = (*list)[entry];
	
	Cmp3ThreadSafeConfig::m_Mutex.Release();

	return result;
}

