/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/
 
// Implementation of class Cmp3WorkerThread

#include "mp3_worker_thread.h"
#include <zz_log_exception.h>
#include <zz_file.h>
#include <zz_win_exception.h>
#include <zz_base64.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <direct.h>
#include "mp3_web_config.h"

CzzWinThreadMutex Cmp3WorkerThread::m_Mutex;

// Construction
Cmp3WorkerThread::Cmp3WorkerThread( CzzAccessLog* log, CzzHTTPRequest* req )
 : CzzHTTPServer(log), m_Log(), m_User(), m_IP(), m_LastRequest(req)
{

}

// Destruction
Cmp3WorkerThread::~Cmp3WorkerThread()
{
	
}

bool Cmp3WorkerThread::checkAuth( CzzHTTPRequest* req )
{
	CzzHTTPHeaders h = req->getHeaders();

	if( h.find( "authorization" ) == h.end() )
	{
		notAuthorized();
		return false;
	}
	else
	{
		CzzString encoded_usrpwd = h[ "authorization" ].substr(6); // Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==
		unsigned char tmp[100];
		memset( tmp, 0, 100 );
		CzzBase64::decode( tmp, 100, encoded_usrpwd.c_str(), encoded_usrpwd.size() );

		// split usr/pwd
		char* usr = strtok( (char*)tmp, ":" );
		char* pwd = strtok( NULL, ":" );
		CzzString pass;

		m_User = usr;

		try{ pass = Cmp3WebConfig::getUserPwd( CzzString(usr) ); } catch( ... ) {}

		if( pass != CzzString(pwd) )
		{
			notAuthorized();
			return false;
		}
		else
			return true;
	}
}

void Cmp3WorkerThread::notAuthorized( void )
{
	CzzHTTPHeaders hdrs;
	CzzBuffer buffer( 1024 );
	CzzString data;

	hdrs[ "date" ] = getDate();
	hdrs[ "server" ] = "libzzinet/1.0";
	hdrs[ "connection" ] = "close";
	hdrs[ "WWW-Authenticate" ] = "Basic realm=\"myMusic\"";

	data = "<html><h1><b>Authorization required</b></h1></html>";
	m_Log.Print( _DEBUGPARAMS, "Sedning: %s", data.c_str() );
	buffer.append( (char*)data.c_str(), data.size() );

	hdrs[ "content-length" ] = CzzString::Format( "%ld", data.size() );

	reply( m_LastRequest, ZZ_HTTP_STATUS_AUTHORIZATION, buffer, hdrs );
}

void Cmp3WorkerThread::OnStart()
{
    try
    {
		if( !checkAuth( m_LastRequest ) )
		{
			m_Running = false;
			return;
		}

		ZZ_HTTP_CMD cmd;
        CzzString uri;
        
        uri = m_LastRequest->getUri();
        cmd = m_LastRequest->getCommand();

		if( uri.find( "?" ) != -1 )
		{
			int pos = uri.find( "?" );
			int s = uri.find( "/cgi-bin/" );

			CzzString params = uri.substr( pos + 1, uri.size() - pos + 1 );
			CzzString script = uri.substr( s+9, pos-(s+9) );
			execute_cgi_script( m_LastRequest, script, params );
		}
		else
			get_file( m_LastRequest, uri );
    }
    catch( CzzException& e )
    {
		if( e.GetCode() == 404 )
			serviceNotFound( e.GetReason().c_str() );
		else
			internalError( e.GetReason().c_str() );
    }
    catch( ... )
    {
        internalError( "Internal server error" );
    }

	m_Running = false;
}

// Execute CGI-script
int Cmp3WorkerThread::execute_cgi_script( CzzHTTPRequest* req, CzzString& script, CzzString& params )
{
	Cmp3WorkerThread::m_Mutex.Acquire( true );

	CzzBuffer buffer( 10240, 10240 );
    CzzString query = CzzString::Format( "QUERY_STRING=%s", params.c_str() );
    int r = 0;
	int size = 0;
	char data[4096];
	char hdr[1024];
	char* p = data;
	bool found = false;
	bool noappend = false;
	int i = 0;
	CzzString cookie;
	char env[4096];
	CzzString envvar;
	int envp = 0;

	CzzHTTPHeaders h = req->getHeaders();

	// TODO:
	m_IP = req->getHostName();

	memset( env, 0, 4096 );

	envvar = CzzString::Format( "SystemRoot=%s", getenv("SystemRoot") );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	envvar = CzzString::Format( "COMSPEC=%s", getenv("COMSPEC") );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	envvar = CzzString::Format( "PATHEXT=%s", getenv("PATHEXT") );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	envvar = CzzString::Format( "WINDIR=%s", getenv("WINDIR") );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	if( h.find( "cookie" ) != h.end() )
	{
		//r = putenv( CzzString::Format( "HTTP_COOKIE=%s", h[ "cookie" ].c_str() ).c_str() );
		envvar = CzzString::Format( "HTTP_COOKIE=%s", h[ "cookie" ].c_str() );
		strcpy( env + envp, envvar.c_str() );
		envp = envp + envvar.size() + 1;
	}

	//r = putenv( "REQUEST_METHOD=GET" );
	envvar = CzzString::Format( "REQUEST_METHOD=%s", "GET" );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	//r = putenv( query.c_str() );
	envvar = CzzString::Format( "%s", query.c_str() );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	//r = putenv( CzzString::Format( "REMOTE_USER=%s", m_User.c_str() ).c_str() );
	envvar = CzzString::Format( "REMOTE_USER=%s", m_User.c_str() );
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	//r = putenv( CzzString::Format( "REMOTE_ADDR=%s", m_IP.c_str() ).c_str() );
	envvar = CzzString::Format( "REMOTE_ADDR=%s", m_IP.c_str());
	strcpy( env + envp, envvar.c_str() );
	envp = envp + envvar.size() + 1;

	if( r == -1 )
	{
		Cmp3WorkerThread::m_Mutex.Release();
		throw CzzLogException( _LAF, 4711, "Couldn't set environment for CGI-script" );
	}

	HANDLE hChildStdoutRd, hChildStdoutWr, hChildStdoutRdDup; 
	HANDLE hChildStdinRd, hChildStdinWr, hChildStdinWrDup;

	SECURITY_ATTRIBUTES sattr;
	PROCESS_INFORMATION pi;
	STARTUPINFO si;
	BOOL rs;
	DWORD read;

	sattr.nLength = sizeof(SECURITY_ATTRIBUTES);
	sattr.bInheritHandle = TRUE;
	sattr.lpSecurityDescriptor = NULL;

	rs = CreatePipe( &hChildStdoutRd, &hChildStdoutWr, &sattr, 0 );
	rs = DuplicateHandle( GetCurrentProcess(), hChildStdoutRd, GetCurrentProcess(), &hChildStdoutRdDup, 0, FALSE, DUPLICATE_SAME_ACCESS );
	rs = CloseHandle( hChildStdoutRd );

	rs = CreatePipe( &hChildStdinRd, &hChildStdinWr, &sattr, 0 );
	rs = DuplicateHandle( GetCurrentProcess(), hChildStdinWr, GetCurrentProcess(), &hChildStdinWrDup, 0, FALSE, DUPLICATE_SAME_ACCESS );
	rs = CloseHandle( hChildStdinWr );

	ZeroMemory( &si, sizeof(STARTUPINFO) );
	ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );

	si.cb = sizeof(STARTUPINFO);
	si.hStdError = hChildStdoutWr;
	si.hStdOutput = hChildStdoutWr;
	si.hStdInput = hChildStdinRd;
	si.dwFlags |= STARTF_USESTDHANDLES;

	rs = createChildProcess( env, script.c_str(), &pi, &si );

	rs = CloseHandle( hChildStdinWrDup );
	rs = CloseHandle( hChildStdoutWr );

	CzzHTTPHeaders hdrs;

	if( rs )
	{
		rs = ReadFile( hChildStdoutRdDup, data, 4096, &read, NULL );

		while( read > 0 && rs )
		{
			// Strip off headers
			while( i < read && !found )
			{
				if( p[ i ] == 13 && p[ i+1 ] == 10 && p[ i+2 ] == 13 && p[ i+3 ] == 10 )
				{
					found = true;
					noappend = true;
					p = p + i + 4;
					
					// copy headers
					memcpy( hdr, data, i );
					hdr[i] = '\0';

					// copy rest of data to buffer
					buffer.append( p, (read - i - 4) );
				}
				
				i++;
			}
			

			if( !noappend )
				buffer.append( p, read );

			noappend = false;
			rs = ReadFile( hChildStdoutRdDup, data, 4096, &read, NULL );
			p = data;
		}

		CloseHandle( hChildStdoutRdDup );
		CloseHandle( hChildStdinRd );

		if( found )
		{
			hdrs[ "Date" ] = getDate();
			hdrs[ "Server" ] = "libzzinet/1.0";
			hdrs[ "Connection" ] = "close";
			hdrs[ "Content-length" ] = CzzString::Format( "%ld", buffer.getSize() );

			// split headers
			char* name = strtok( hdr, ":\n" );
			char* value = NULL;

			while( name != NULL )
			{
				value = strtok( NULL, ":\n" );

				m_Log.Print( _TRACEPARAMS, "Adding header from CGI: '%s: %s'", trim(name).c_str(), trim(value).c_str() );

				hdrs[ trim(name) ] = trim(value);
				name = strtok( NULL, ":\n" );
			}

			reply( m_LastRequest, ZZ_HTTP_STATUS_OK, buffer, hdrs );
		}
		else
			reply( m_LastRequest, ZZ_HTTP_STATUS_OK, ZZ_HTTP_CONTENT_TYPE_HTML, buffer );
	}
	else
	{
		Cmp3WorkerThread::m_Mutex.Release();
		throw CzzLogException( _LAF, 4711, CzzString::Format( "Couldn't execute script %s (errno=%d)", script.c_str(), errno ) );
	}

	Cmp3WorkerThread::m_Mutex.Release();

    return buffer.getSize();
}

// Get file
int Cmp3WorkerThread::get_file( CzzHTTPRequest* req, CzzString& uri )
{
	CzzString localurl = translateToLocal( uri );
	CzzBuffer buffer( 10240, 10240 );
	CzzFile file( localurl.c_str() );
	ZZ_HTTP_CONTENT_TYPE content;

	try
	{
		file.Open( "rb" );
	}
	catch( ... )
	{
		if( uri.find( "folder.jpg" ) != -1 || uri.find( "Folder.jpg" ) != -1 )
		{
			file.SetPath( CzzString::Format( "%s\\404.jpg", Cmp3WebConfig::getDocumentRoot().c_str() ).c_str() );
			file.Open( "rb" );
		}
		else
			throw CzzLogException( _LAF, 404, CzzString::Format( "'%s' not found", uri.c_str() ) );
	}

	size_t size = file.Size();
	char* data = new char[ size ];
	file.Read( data, size );
	file.Close();
	buffer.append( data, size );
	delete [] data;

	if( uri.find( ".gif" ) != -1 )
		content = ZZ_HTTP_CONTENT_TYPE_GIF;
	if( uri.find( ".m3u" ) != -1 )
		content = ZZ_HTTP_CONTENT_TYPE_M3U;
	if( uri.find( ".css" ) != -1 )
		content = ZZ_HTTP_CONTENT_TYPE_CSS;
	else if( uri.find( ".jpg" ) != -1 )
		content = ZZ_HTTP_CONTENT_TYPE_JPEG;
	else if( uri.find( ".htm" ) != -1 )
		content = ZZ_HTTP_CONTENT_TYPE_HTML;
	else
		content = ZZ_HTTP_CONTENT_TYPE_TEXT;

	reply( m_LastRequest, ZZ_HTTP_STATUS_OK, content, buffer );

	return buffer.getSize();
};

CzzString Cmp3WorkerThread::translateToLocal( CzzString& uri )
{
	if( uri[ uri.size()-1 ] == '/' )
		uri = uri + "index.html";

	CzzString result;
	int p = 0;
	CzzString root = Cmp3WebConfig::getDocumentRoot();
	CzzString localdir;
	CzzString rest;
	CzzString alias;

	for( int i=1; i < uri.size(); i++ )
	{
		if( uri[i] == '/' )
		{
			p = i;
			break;
		}
	}

	if( p == 0 )
		p = 1;

	alias = uri.substr( 0, p );

	if( alias == "/" )
		localdir = root + "\\";
	else
		localdir = Cmp3WebConfig::getAlias( alias );

	rest = uri.substr( p, uri.size() - p );


	CzzString local;

	// Try localized and skinified URL
	if( localdir[0] == '\\' )
		local = CzzString::Format( "%s%s\\%s\\%s%s", root.c_str(), localdir.c_str(), Cmp3WebConfig::getLang().c_str(), Cmp3WebConfig::getSkin().c_str(), rest.c_str() );
	else
		local = CzzString::Format( "%s\\%s\\%s%s", localdir.c_str(), Cmp3WebConfig::getLang().c_str(), Cmp3WebConfig::getSkin().c_str(), rest.c_str() );

	local = urlDecode( local );

	m_Log.Print( _TRACEPARAMS, "SKIN: %s", local.c_str() );

	// Check special language and skin file
	CzzFile file( local.c_str() );

	if( file.Exist() )
		result = local;
	else
	{
		// Try localized URL
		if( localdir[0] == '\\' )
			local = CzzString::Format( "%s%s\\%s%s", root.c_str(), localdir.c_str(), Cmp3WebConfig::getLang().c_str(), rest.c_str() );
		else
			local = CzzString::Format( "%s\\%s%s", localdir.c_str(), Cmp3WebConfig::getLang().c_str(), rest.c_str() );

		local = urlDecode( local );

		m_Log.Print( _TRACEPARAMS, "LANG: %s", local.c_str() );

		// Check special language file
		CzzFile file( local.c_str() );

		if( file.Exist() )
			result = local;
		else
		{
			// Default to no language or skin specific file
			if( localdir[0] == '\\' )
				result = CzzString::Format( "%s%s%s", root.c_str(), localdir.c_str(), rest.c_str() );
			else
				result = CzzString::Format( "%s%s", localdir.c_str(), rest.c_str() );

			result = urlDecode( result );
		}
	}

	return result;
}

// Service not found error reply
int Cmp3WorkerThread::serviceNotFound( const char* msg )
{
    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "<html><h1><b>%s</b></h1></html>", msg );

    m_Log.Print( _DEBUGPARAMS, "Sending: %s", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( m_LastRequest, ZZ_HTTP_STATUS_NOT_FOUND, ZZ_HTTP_CONTENT_TYPE_HTML, buffer );
    
    return buffer.getSize();
}

// Internal error reply
int Cmp3WorkerThread::internalError( const char* msg )
{
    CzzBuffer buffer( 1024 );
    CzzString data;

    data = CzzString::Format( "<html><h1><b>%s</b></h1></html>", msg );

    m_Log.Print( _DEBUGPARAMS, "Sending: %s", data.c_str() );

    buffer.append( (char*)data.c_str(), data.size() );
    reply( m_LastRequest, ZZ_HTTP_STATUS_INTERNAL_SERVER_ERROR, ZZ_HTTP_CONTENT_TYPE_HTML, buffer );

    return buffer.getSize();
}

// Get date
CzzString Cmp3WorkerThread::getDate( void )
{
	time_t now;
	tm	*lt;
    time( &now );
	lt = localtime( &now );
    return CzzString::Format( "%02d-%02d-%04d",lt->tm_mday, lt->tm_mon+1, lt->tm_year+1900 );
}

CzzString Cmp3WorkerThread::trim( const CzzString& in )
{
	CzzString result = in;
	int p = 0;

	while( (p = result.find( " " )) != -1 )
		result.replace( p, 1, "" );
	while( (p = result.find( (char)13 )) != -1 )
		result.replace( p, 1, "" );

	return result;
}

CzzString Cmp3WorkerThread::urlDecode( const CzzString& url )
{
	CzzString result = url;
	int p = 0;

	while( (p = result.find( "%20" )) != -1 )
		result.replace( p, 3, " " );

	while( (p = result.find( "%60" )) != -1 )
		result.replace( p, 3, "`" );

	while( (p = result.find( "%C3%85" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%84" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%96" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%A5" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%A4" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%B6" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%27" )) != -1 )
		result.replace( p, 3, "'" );

	while( (p = result.find( "%26" )) != -1 )
		result.replace( p, 3, "&" );

	while( (p = result.find( "%2C" )) != -1 )
		result.replace( p, 3, "," );

	while( (p = result.find( "%21" )) != -1 )
		result.replace( p, 3, "!" );

	while( (p = result.find( "%28" )) != -1 )
		result.replace( p, 3, "(" );

	while( (p = result.find( "%29" )) != -1 )
		result.replace( p, 3, ")" );

    while( (p = result.find( "%5B" )) != -1 )
		result.replace( p, 3, "[" );

	while( (p = result.find( "%5D" )) != -1 )
		result.replace( p, 3, "]" );

	while( (p = result.find( "%C3%BC" )) != -1 )
		result.replace( p, 6, "" );

	while( (p = result.find( "%C3%9C" )) != -1 )
		result.replace( p, 6, "" );

	return result;
}

bool Cmp3WorkerThread::createChildProcess( char* env, const char* cmd, PROCESS_INFORMATION* pinfo, STARTUPINFO* sinfo )
{
	bool result = false;
	BOOL r = FALSE;


	if( CreateProcess( NULL, (char*)cmd, NULL, NULL, TRUE, 0, env, NULL, sinfo, pinfo ) )
	{
		CloseHandle( pinfo->hProcess );
		CloseHandle( pinfo->hThread );
		result = true;
	}
	else
		result = false;

	return result;
}