/* libzzthread
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2000-2006, Toni Thomsson <toni@tonjac.org> 
*/

#ifndef zz_win_thread_h
#define zz_win_thread_h

#include <windows.h>
#include <zz_win_exception.h>

// Description: Exitkod som talar om att trden "ddats hrt"
#define ZZ_THREAD_FORCED_STOP	0xFFFFFFFF

/**
	Windows thread.

	Remarks:
	Subklassa och implementera OnStart(). Skapa ett objekt av din klass
	och anropa Run() fr att exekvera trden, Run() anropar OnStart().
	Trden avslutas nr OnStart() har exekverat frdigt eller nr Stop()
	anropas.

	@author Toni Thomsson, toni@tonjac.org
*/
class ZZ_API CzzWinThread
{
public:

	/* 
		Description:
		Skapar en trd

		Throws:
		CzzException
	*/
	CzzWinThread();

    CzzWinThread( LPTHREAD_START_ROUTINE startf );
    
	// Remarks: Skapar en trd med viss prioritet
	CzzWinThread( int prio );

	virtual ~CzzWinThread();

	// Description: Exekvera trden
	// Throws: CzzException
	void Run( void );

	// Description: Kontrollerar om trden exekverar
	bool Running( void );

	// Description: Stoppa trden tillflligt
	void Suspend( void );

	// Description: Starta trden igen
	void Resume( void );

	/* 
		Description: 
		Stoppa trden permanent

		Remarks:
		Trden kan stoppas mjukt: den fr gra klart det den hller p med
		och sjlv avgra nr den skall avslutas.
		Trden kan stoppas hrt: avbryter oavsett vad trden gr
	*/
	void Stop( bool hard,		// IN, stoppflagga
				DWORD exitcode	// IN, exitkod
				);

	// Description: Exitkod
	DWORD ExitCode( void ) { return m_ExitCode; };

	// Description: Prioritet
	void SetPrio( int prio );
		
	/*
		Description:
		Anropas nr trden startar, jfr. ett vanligt programs main()

		Remarks:
		Skall implementeras i subklassen (verlagras) default
		implementationen gr inte ngot mer n stter m_Running = false
	*/
	virtual void OnStart( void ) { m_Running = false;
									m_ExitCode = 0; };
	
protected:

	// Description: Exitkod
	DWORD m_ExitCode;

	// Description: Flagga som talar om att trden exekverat frdigt
	bool m_Running;

	// Description: Flagga som talar om att ngon begrt att trden skall stoppas
	bool m_StopRequest;

	unsigned long m_Id;
	DWORD m_SuspendCount;
	HANDLE m_Handle;
	int m_Prio;

	LPTHREAD_START_ROUTINE m_StartFunction;
 
private:
	static unsigned int WINAPI Begin( LPVOID param );

    void Cleanup(void);
};

#endif // zz_win_thread_h